@@ -1,11 +1,13 @@
 from __future__ import annotations
 from typing import Callable, NoReturn, Optional, Dict, List, Set, Any, Tuple, Union, Type, TypeVar, Generic
 
+import json
 import logging
-from datetime import datetime, timezone
+import time as timemodule
+from datetime import datetime, timezone, time
 from enum import Enum, IntEnum, Flag, auto
 
-from weconnect.util import toBool, imgToASCIIArt
+from weconnect.util import toBool, imgToASCIIArt, robustTimeParse, ExtendedWithNullEncoder
 
 SUPPORT_IMAGES = False
 try:
@@ -182,22 +184,38 @@ class AddressableAttribute(AddressableLeaf, Generic[T]):
         parent: AddressableObject,
         value: Optional[T],
         valueType: Type[T],
-        lastUpdateFromCar: Optional[datetime] = None
+        lastUpdateFromCar: Optional[datetime] = None,
+        valueGetter: Optional[Callable[[], Optional[T]]] = None,
+        valueSetter: Optional[Callable[[Optional[T]], None]] = None
     ) -> None:
         super().__init__(localAddress, parent)
         self.__value: Optional[T] = None
         self.valueType: Type[T] = valueType
+        self.valueGetter = valueGetter
+        self.valueSetter = valueSetter
         if value is not None:
             self.setValueWithCarTime(value, lastUpdateFromCar, fromServer=True)
 
     @property
     def value(self) -> Optional[T]:
+        if self.valueGetter is not None:
+            return self.valueGetter()
         return self.__value
 
     @value.setter
     def value(self, newValue: Optional[T]) -> NoReturn:
         raise NotImplementedError('You cannot set this attribute. Set is not implemented')
 
+    def asDict(self, filterCallable: Optional[Callable[[Any], None]] = None):
+        if filterCallable is None or not filterCallable(self.value):
+            return self.value
+        return None
+
+    def toJSON(self):
+        if SUPPORT_IMAGES and isinstance(self.value, Image.Image):
+            return None
+        return json.dumps(self.value, cls=ExtendedWithNullEncoder, skipkeys=True, indent=4)
+
     def setValueWithCarTime(self, newValue, lastUpdateFromCar: Optional[datetime] = None, fromServer: bool = False, noNotify: bool = False) -> None:
         if newValue is not None and not isinstance(newValue, self.valueType):
             raise ValueError(f'{self.getGlobalAddress()}: new value {newValue} must be of type {self.valueType}'
@@ -205,7 +223,7 @@ class AddressableAttribute(AddressableLeaf, Generic[T]):
         valueChanged: bool = newValue != self.__value
         self.__value = newValue
         flags: Optional[AddressableLeaf.ObserverEvent] = None
-        if not self.enabled:
+        if not self.enabled and valueChanged:
             self.enabled = True
         self.lastUpdateFromServer = datetime.utcnow().replace(microsecond=0, tzinfo=timezone.utc)
         if valueChanged:
@@ -232,9 +250,6 @@ class AddressableAttribute(AddressableLeaf, Generic[T]):
     def isLeaf(self) -> bool:  # pylint: disable=R0201
         return True
 
-    def getLeafChildren(self) -> List[AddressableLeaf]:
-        return [self]
-
     def saveToFile(self, filename: str) -> None:  # noqa: C901
         if self.value is not None:
             if filename.endswith(('.txt', '.TXT', '.text')):
@@ -253,6 +268,11 @@ class AddressableAttribute(AddressableLeaf, Generic[T]):
                         htmlfile.write('<pre/></body></html>')
                     else:
                         htmlfile.write(str(self))
+            elif filename.endswith(('.json')):
+                with open(filename, mode='w', encoding='utf8') as textfile:
+                    if SUPPORT_IMAGES and SUPPORT_ASCII_IMAGES and isinstance(self.value, Image.Image):
+                        raise ValueError('Attribute is an image and cannot be converted to json')
+                    textfile.write(self.toJSON() + '\n')
             elif filename.endswith(('.png', '.PNG')):
                 with open(filename, mode='wb') as pngfile:
                     if SUPPORT_IMAGES and isinstance(self.value, Image.Image):
@@ -272,6 +292,37 @@ class AddressableAttribute(AddressableLeaf, Generic[T]):
         else:
             raise ValueError('I cannot save None value')
 
+    def fromDict(self, fromDict: dict, key: str):  # noqa: C901
+        if fromDict is not None and key in fromDict and fromDict[key] is not None:
+            if issubclass(self.valueType, bool):
+                self.setValueWithCarTime(toBool(fromDict[key]), lastUpdateFromCar=None, fromServer=True)
+            elif issubclass(self.valueType, int):
+                self.setValueWithCarTime(int(fromDict[key]), lastUpdateFromCar=None, fromServer=True)
+            elif issubclass(self.valueType, float):
+                self.setValueWithCarTime(float(fromDict[key]), lastUpdateFromCar=None, fromServer=True)
+            elif issubclass(self.valueType, Enum):
+                if fromDict[key]:
+                    try:
+                        self.setValueWithCarTime(self.valueType(fromDict[key]), lastUpdateFromCar=None, fromServer=True)
+                    except ValueError:
+                        self.setValueWithCarTime(self.valueType.UNKNOWN, lastUpdateFromCar=None, fromServer=True)
+                        LOG.warning('%s: An unsupported %s: %s was provided, known values are [%s]'
+                                    ' please report this as a bug', self.getGlobalAddress(), key, fromDict[key],
+                                    ', '.join([state.value for state in list(self.valueType)]))
+                else:
+                    self.enabled = False
+            elif issubclass(self.valueType, datetime):
+                self.setValueWithCarTime(robustTimeParse(fromDict[key]), lastUpdateFromCar=None, fromServer=True)
+            elif issubclass(self.valueType, time):
+                parsedtime = timemodule.strptime(fromDict[key], "%H:%M")
+                self.setValueWithCarTime(time(hour=parsedtime.tm_hour, minute=parsedtime.tm_min), lastUpdateFromCar=None, fromServer=True)
+            elif issubclass(self.valueType, str):
+                self.setValueWithCarTime(str(fromDict[key]), lastUpdateFromCar=None, fromServer=True)
+            else:
+                raise ValueError(f'Unknown attribute type {self.valueType}')
+        else:
+            self.enabled = False
+
     def __str__(self) -> str:
         if isinstance(self.value, Enum):
             return str(self.value.value)  # pylint: disable=no-member
@@ -290,9 +341,11 @@ class ChangeableAttribute(AddressableAttribute):
         value,
         valueType=str,
         lastUpdateFromCar: Optional[datetime] = None,
+        valueGetter: Optional[Callable[[], Optional[T]]] = None,
+        valueSetter: Optional[Callable[[Optional[T]], None]] = None
     ) -> None:
         super().__init__(localAddress=localAddress, parent=parent, value=value, valueType=valueType,
-                         lastUpdateFromCar=lastUpdateFromCar)
+                         lastUpdateFromCar=lastUpdateFromCar, valueGetter=valueGetter, valueSetter=valueSetter)
 
     @AddressableAttribute.value.setter  # type: ignore # noqa: C901
     def value(self, newValue):  # noqa: C901
@@ -331,7 +384,10 @@ class ChangeableAttribute(AddressableAttribute):
             if self.valueType == float:
                 newValue = float(newValue)
 
-        self.setValueWithCarTime(newValue=newValue, lastUpdateFromCar=None)
+        if self.valueSetter is not None:
+            self.valueSetter(newValue)
+        else:
+            self.setValueWithCarTime(newValue=newValue, lastUpdateFromCar=None)
 
 
 class AliasChangeableAttribute(ChangeableAttribute):
@@ -375,6 +431,22 @@ class AddressableObject(AddressableLeaf):
                 child.enabled = False
         AddressableLeaf.enabled.fset(self, setEnabled)  # type: ignore
 
+    def asDict(self, filterCallable: Optional[Callable[[Any], None]] = None):
+        asDict = {}
+        for child in self.children:
+            if child.enabled:
+                childDict = child.asDict(filterCallable=filterCallable)
+                if childDict is not None:
+                    asDict[child.getLocalAddress()] = childDict
+        return asDict
+
+    def toJSON(self):
+        def filterDict(element):
+            if SUPPORT_IMAGES and isinstance(element, Image.Image):
+                return True
+            return False
+        return json.dumps(self.asDict(filterCallable=filterDict), cls=ExtendedWithNullEncoder, skipkeys=True, indent=4)
+
     def isLeaf(self) -> bool:
         return not self.__children
 
@@ -385,17 +457,23 @@ class AddressableObject(AddressableLeaf):
         self.enabled = True
 
     def getLeafChildren(self) -> List[AddressableLeaf]:
+        return self.getRecursiveChildren(leaveOnly=True)
+
+    def getRecursiveChildren(self, leaveOnly=False) -> List[AddressableLeaf]:
         if not self.enabled:
             return []
         if self.isLeaf():
             return [self]
 
-        children: List[AddressableLeaf] = [self]
+        children: List[AddressableLeaf] = []
+        if not leaveOnly:
+            children.append(self)
         for child in self.__children.values():
-            if isinstance(child, AddressableObject):
-                children.extend(child.getLeafChildren())
-            elif isinstance(child, AddressableLeaf) and child.enabled:
-                children.append(child)
+            if child.enabled:
+                if isinstance(child, AddressableObject):
+                    children.extend(child.getRecursiveChildren(leaveOnly=leaveOnly))
+                elif isinstance(child, AddressableLeaf):
+                    children.append(child)
         return children
 
     @property
@@ -423,6 +501,25 @@ class AddressableObject(AddressableLeaf):
             child.updateComplete()
         super().updateComplete()
 
+    def saveToFile(self, filename: str) -> None:  # noqa: C901
+        if filename.endswith(('.txt', '.TXT', '.text')):
+            with open(filename, mode='w', encoding='utf8') as textfile:
+                textfile.write(str(self))
+        elif filename.endswith(('.htm', '.HTM', '.html', '.HTML')):
+            with open(filename, mode='w', encoding='utf8'):
+                raise ValueError('Object cannot be saved as HTML')
+        elif filename.endswith(('.json')):
+            with open(filename, mode='w', encoding='utf8') as textfile:
+                textfile.write(self.toJSON() + '\n')
+        elif filename.endswith(('.png', '.PNG')):
+            with open(filename, mode='wb'):
+                raise ValueError('Attribute is no image and cannot be converted to one')
+        elif filename.endswith(('.jpg', '.JPG', '.jpeg', '.JPEG')):
+            with open(filename, mode='wb'):
+                raise ValueError('Attribute is no image and cannot be converted to one')
+        else:
+            raise ValueError('I cannot recognize the target file extension')
+
 
 L = TypeVar('L', bound=AddressableLeaf)
 
