@@ -33,45 +33,36 @@ class ChargingStatus(GenericStatus):
         LOG.debug('Update Charging status from dict')
 
         if 'value' in fromDict:
-            if 'remainingChargingTimeToComplete_min' in fromDict['value']:
-                self.remainingChargingTimeToComplete_min \
-                    .setValueWithCarTime(int(fromDict['value']['remainingChargingTimeToComplete_min']), lastUpdateFromCar=None,
-                                         fromServer=True)
-            else:
-                self.remainingChargingTimeToComplete_min.enabled = False
-
-            if 'chargingState' in fromDict['value'] and fromDict['value']['chargingState']:
-                try:
-                    self.chargingState.setValueWithCarTime(ChargingStatus.ChargingState(fromDict['value']['chargingState']),
-                                                           lastUpdateFromCar=None)
-                except ValueError:
-                    self.chargingState.setValueWithCarTime(
-                        ChargingStatus.ChargingState.UNKNOWN, lastUpdateFromCar=None, fromServer=True)
-                    LOG.warning('An unsupported chargingState: %s was provided,'
-                                ' please report this as a bug', fromDict['value']['chargingState'])
-            else:
-                self.chargingState.enabled = False
-
-            if 'chargeMode' in fromDict['value'] and fromDict['value']['chargeMode']:
-                try:
-                    self.chargeMode.setValueWithCarTime(ChargingStatus.ChargeMode(fromDict['value']['chargeMode']), lastUpdateFromCar=None)
-                except ValueError:
-                    self.chargeMode.setValueWithCarTime(
-                        ChargingStatus.ChargeMode.UNKNOWN, lastUpdateFromCar=None, fromServer=True)
-                    LOG.warning('An unsupported chargeMode: %s was provided,'
-                                ' please report this as a bug', fromDict['value']['chargeMode'])
-            else:
-                self.chargeMode.enabled = False
-
+            self.remainingChargingTimeToComplete_min.fromDict(fromDict['value'], 'remainingChargingTimeToComplete_min')
+            self.chargingState.fromDict(fromDict['value'], 'chargingState')
+            self.chargeMode.fromDict(fromDict['value'], 'chargeMode')
+            self.chargePower_kW.fromDict(fromDict['value'], 'chargePower_kW')
             if 'chargePower_kW' in fromDict['value']:
-                self.chargePower_kW.setValueWithCarTime(
-                    float(fromDict['value']['chargePower_kW']), lastUpdateFromCar=None, fromServer=True)
+                chargePower_kW = float(fromDict['value']['chargePower_kW'])
+                if self.fixAPI and chargePower_kW != 0 \
+                        and self.chargingState.value in [ChargingStatus.ChargingState.OFF,
+                                                         ChargingStatus.ChargingState.READY_FOR_CHARGING,
+                                                         ChargingStatus.ChargingState.NOT_READY_FOR_CHARGING,
+                                                         ChargingStatus.ChargingState.CHARGE_PURPOSE_REACHED_NOT_CONSERVATION_CHARGING,
+                                                         ChargingStatus.ChargingState.ERROR]:
+                    chargePower_kW = 0.0
+                    LOG.debug('%s: Attribute chargePower_kW is %s while chargingState is %s. Setting 0 instead',
+                              self.getGlobalAddress(), fromDict['value']['chargePower_kW'], self.climatisationState.value)
+                self.chargePower_kW.setValueWithCarTime(chargePower_kW, lastUpdateFromCar=None, fromServer=True)
             else:
                 self.chargePower_kW.enabled = False
-
             if 'chargeRate_kmph' in fromDict['value']:
-                self.chargeRate_kmph.setValueWithCarTime(
-                    float(fromDict['value']['chargeRate_kmph']), lastUpdateFromCar=None, fromServer=True)
+                chargeRate_kmph = float(fromDict['value']['chargeRate_kmph'])
+                if self.fixAPI and chargeRate_kmph != 0 \
+                        and self.chargingState.value in [ChargingStatus.ChargingState.OFF,
+                                                         ChargingStatus.ChargingState.READY_FOR_CHARGING,
+                                                         ChargingStatus.ChargingState.NOT_READY_FOR_CHARGING,
+                                                         ChargingStatus.ChargingState.CHARGE_PURPOSE_REACHED_NOT_CONSERVATION_CHARGING,
+                                                         ChargingStatus.ChargingState.ERROR]:
+                    chargeRate_kmph = 0.0
+                    LOG.debug('%s: Attribute chargeRate_kmph is %s while chargingState is %s. Setting 0 instead',
+                              self.getGlobalAddress(), fromDict['value']['chargeRate_kmph'], self.climatisationState.value)
+                self.chargeRate_kmph.setValueWithCarTime(chargeRate_kmph, lastUpdateFromCar=None, fromServer=True)
             else:
                 self.chargeRate_kmph.enabled = False
         else:
