@@ -89,21 +89,37 @@ class GenericStatus(AddressableObject):
             self.error.reset()
 
         if 'requests' in fromDict:
+            requestsToRemove = self.requests.copy()
             for request in fromDict['requests']:
                 updated = False
                 if 'requestId' in request:
                     for knownRequest in self.requests:
                         if knownRequest.requestId.enabled and knownRequest.requestId.value == request['requestId']:
-                            request.update(fromDict=request)
+                            knownRequest.update(fromDict=request)
+                            if knownRequest in requestsToRemove:
+                                requestsToRemove.remove(knownRequest)
                             updated = True
                 elif 'operation' in request:
                     for knownRequest in self.requests:
                         if knownRequest.operation.enabled and knownRequest.operation.value.value == request['operation']:
-                            request.update(fromDict=request)
+                            knownRequest.update(fromDict=request)
+                            if knownRequest in requestsToRemove:
+                                requestsToRemove.remove(knownRequest)
                             updated = True
 
                 if not updated:
                     self.requests.append(GenericStatus.Request(localAddress=str(len(self.requests)), parent=self.requests, fromDict=request))
+            for request in requestsToRemove:
+                self.requests.remove(request)
+
+            index = 0
+            for request in self.requests:
+                if request != str(index):
+                    request.enabled = False
+                    request.localAddress = str(index)
+                    request.enabled = True
+                index += 1
+
         else:
             self.requests.clear()
             self.requests.enabled = False
@@ -145,55 +161,13 @@ class GenericStatus(AddressableObject):
 
         def update(self, fromDict: Dict[str, Any]) -> None:  # noqa: C901
             LOG.debug('Update Status Target from dict')
-
-            if 'status' in fromDict and fromDict['status']:
-                try:
-                    self.status.setValueWithCarTime(GenericStatus.Request.Status(
-                        fromDict['status']), lastUpdateFromCar=None, fromServer=True)
-                except ValueError:
-                    self.status.setValueWithCarTime(GenericStatus.Request.Status.UNKNOWN,
-                                                    lastUpdateFromCar=None, fromServer=True)
-                    LOG.warning('An unsupported target status: %s was provided,'
-                                ' please report this as a bug', fromDict['status'])
-            else:
-                self.status.enabled = False
-
-            if 'operation' in fromDict and fromDict['operation']:
-                try:
-                    self.operation.setValueWithCarTime(ControlOperation(
-                        fromDict['operation']), lastUpdateFromCar=None, fromServer=True)
-                except ValueError:
-                    self.operation.setValueWithCarTime(ControlOperation.UNKNOWN,
-                                                       lastUpdateFromCar=None, fromServer=True)
-                    LOG.warning('An unsupported target operation: %s was provided,'
-                                ' please report this as a bug', fromDict['operation'])
-            else:
-                self.operation.enabled = False
-
-            if 'body' in fromDict:
-                self.body.setValueWithCarTime(str(fromDict['body']), lastUpdateFromCar=None, fromServer=True)
-            else:
-                self.body.enabled = False
-
-            if 'group' in fromDict:
-                self.group.setValueWithCarTime(int(fromDict['group']), lastUpdateFromCar=None, fromServer=True)
-            else:
-                self.group.enabled = False
-
-            if 'info' in fromDict:
-                self.info.setValueWithCarTime(fromDict['info'], lastUpdateFromCar=None, fromServer=True)
-            else:
-                self.info.enabled = False
-
-            if 'requestId' in fromDict:
-                self.requestId.setValueWithCarTime(fromDict['requestId'], lastUpdateFromCar=None, fromServer=True)
-            else:
-                self.requestId.enabled = False
-
-            if 'vcfRequestId' in fromDict:
-                self.vcfRequestId.setValueWithCarTime(fromDict['vcfRequestId'], lastUpdateFromCar=None, fromServer=True)
-            else:
-                self.vcfRequestId.enabled = False
+            self.status.fromDict(fromDict, 'status')
+            self.operation.fromDict(fromDict, 'operation')
+            self.body.fromDict(fromDict, 'body')
+            self.group.fromDict(fromDict, 'group')
+            self.info.fromDict(fromDict, 'info')
+            self.requestId.fromDict(fromDict, 'requestId')
+            self.vcfRequestId.fromDict(fromDict, 'vcfRequestId')
 
         def __str__(self) -> str:
             returnValue: str = ''
@@ -219,4 +193,5 @@ class GenericStatus(AddressableObject):
             FAIL_IGNITION_ON = 'fail_ignition_on'
             FAIL_BATTERY_LOW = 'fail_battery_low'
             FAIL_PLUG_ERROR = 'fail_plug_error'
+            FAIL_CHARGE_PLUG_NOT_CONNECTED = 'fail_charge_plug_not_connected'
             UNKNOWN = 'unknown status'
