@@ -1,5 +1,5 @@
 from __future__ import annotations
-from typing import Dict, Set, Any, Type, Optional, cast, TYPE_CHECKING
+from typing import Dict, List, Set, Any, Type, Optional, cast, TYPE_CHECKING
 import os
 from enum import Enum
 from datetime import datetime, timedelta
@@ -88,6 +88,7 @@ class Vehicle(AddressableObject):  # pylint: disable=too-many-instance-attribute
         self.capabilities: AddressableDict[str, GenericCapability] = AddressableDict(localAddress='capabilities', parent=self)
         self.domains: AddressableDict[str, AddressableDict[str, GenericStatus]] = AddressableDict(localAddress='domains', parent=self)
         self.images: AddressableAttribute[Dict[str, str]] = AddressableAttribute(localAddress='images', parent=self, value=None, valueType=dict)
+        self.tags: AddressableAttribute[List[str]] = AddressableAttribute(localAddress='tags', parent=self, value=None, valueType=list)
         self.coUsers: AddressableList[Vehicle.User] = AddressableList(localAddress='coUsers', parent=self)
         self.controls: Controls = Controls(localAddress='controls', vehicle=self, parent=self)
         self.fixAPI: bool = fixAPI
@@ -126,60 +127,14 @@ class Vehicle(AddressableObject):  # pylint: disable=too-many-instance-attribute
     ) -> None:
         if fromDict is not None:
             LOG.debug('Create /update vehicle')
-            if 'vin' in fromDict:
-                self.vin.setValueWithCarTime(fromDict['vin'], lastUpdateFromCar=None, fromServer=True)
-            else:
-                self.vin.enabled = False
 
-            if 'role' in fromDict and fromDict['role']:
-                try:
-                    self.role.setValueWithCarTime(Vehicle.User.Role(fromDict['role']), lastUpdateFromCar=None, fromServer=True)
-                except ValueError:
-                    self.role.setValueWithCarTime(Vehicle.User.Role.UNKNOWN, lastUpdateFromCar=None, fromServer=True)
-                    LOG.warning('An unsupported role: %s was provided, please report this as a bug', fromDict['role'])
-            else:
-                self.role.enabled = False
-
-            if 'enrollmentStatus' in fromDict and fromDict['enrollmentStatus']:
-                try:
-                    self.enrollmentStatus.setValueWithCarTime(Vehicle.User.EnrollmentStatus(fromDict['enrollmentStatus']), lastUpdateFromCar=None,
-                                                              fromServer=True)
-                    if self.enrollmentStatus.value == Vehicle.User.EnrollmentStatus.GDC_MISSING:
-                        LOG.warning('WeConnect reported enrollmentStatus GDC_MISSING. This means you have to login at'
-                                    ' myvolkswagen.de website and accept the terms and conditions')
-                except ValueError:
-                    self.enrollmentStatus.setValueWithCarTime(Vehicle.User.EnrollmentStatus.UNKNOWN, lastUpdateFromCar=None, fromServer=True)
-                    LOG.warning('An unsupported enrollment Status: %s was provided, please report this as a bug', fromDict['enrollmentStatus'])
-            else:
-                self.enrollmentStatus.enabled = False
-
-            if 'userRoleStatus' in fromDict and fromDict['userRoleStatus']:
-                try:
-                    self.userRoleStatus.setValueWithCarTime(Vehicle.User.UserRoleStatus(fromDict['userRoleStatus']), lastUpdateFromCar=None, fromServer=True)
-                except ValueError:
-                    self.userRoleStatus.setValueWithCarTime(Vehicle.User.UserRoleStatus.UNKNOWN, lastUpdateFromCar=None, fromServer=True)
-                    LOG.warning('An unsupported userRoleStatus: %s was provided, please report this as a bug', fromDict['userRoleStatus'])
-            else:
-                self.userRoleStatus.enabled = False
-
-            if 'model' in fromDict:
-                self.model.setValueWithCarTime(fromDict['model'], lastUpdateFromCar=None, fromServer=True)
-            else:
-                self.model.enabled = False
-
-            if 'devicePlatform' in fromDict and fromDict['devicePlatform']:
-                try:
-                    self.devicePlatform.setValueWithCarTime(Vehicle.DevicePlatform(fromDict['devicePlatform']), lastUpdateFromCar=None, fromServer=True)
-                except ValueError:
-                    self.devicePlatform.setValueWithCarTime(Vehicle.DevicePlatform.UNKNOWN, lastUpdateFromCar=None, fromServer=True)
-                    LOG.warning('An unsupported devicePlatform: %s was provided, please report this as a bug', fromDict['devicePlatform'])
-            else:
-                self.devicePlatform.enabled = False
-
-            if 'nickname' in fromDict:
-                self.nickname.setValueWithCarTime(fromDict['nickname'], lastUpdateFromCar=None, fromServer=True)
-            else:
-                self.nickname.enabled = False
+            self.vin.fromDict(fromDict, 'vin')
+            self.role.fromDict(fromDict, 'role')
+            self.enrollmentStatus.fromDict(fromDict, 'enrollmentStatus')
+            self.userRoleStatus.fromDict(fromDict, 'userRoleStatus')
+            self.model.fromDict(fromDict, 'model')
+            self.devicePlatform.fromDict(fromDict, 'devicePlatform')
+            self.nickname.fromDict(fromDict, 'nickname')
 
             if updateCapabilities and 'capabilities' in fromDict and fromDict['capabilities'] is not None:
                 for capDict in fromDict['capabilities']:
@@ -203,6 +158,11 @@ class Vehicle(AddressableObject):  # pylint: disable=too-many-instance-attribute
             else:
                 self.images.enabled = False
 
+            if 'tags' in fromDict:
+                self.tags.setValueWithCarTime(fromDict['tags'], lastUpdateFromCar=None, fromServer=True)
+            else:
+                self.tags.enabled = False
+
             if 'coUsers' in fromDict and fromDict['coUsers'] is not None:
                 for user in fromDict['coUsers']:
                     if 'id' in user:
@@ -230,6 +190,7 @@ class Vehicle(AddressableObject):  # pylint: disable=too-many-instance-attribute
                                               'nickname',
                                               'capabilities',
                                               'images',
+                                              'tags',
                                               'coUsers']}.items():
                 LOG.warning('%s: Unknown attribute %s with value %s', self.getGlobalAddress(), key, value)
 
@@ -319,7 +280,7 @@ class Vehicle(AddressableObject):  # pylint: disable=too-many-instance-attribute
                     continue
                 if domain.value in data:
                     if domain.value not in self.domains:
-                        self.domains[domain.value] = AddressableDict(localAddress=domain.value, parent=self)
+                        self.domains[domain.value] = AddressableDict(localAddress=domain.value, parent=self.domains)
                     for key, className in keyClassMap.items():
                         if key in data[domain.value]:
                             if key in self.domains[domain.value]:
@@ -341,7 +302,8 @@ class Vehicle(AddressableObject):  # pylint: disable=too-many-instance-attribute
         # Controls
         self.controls.update()
 
-        if not updateCapabilities or ('parkingPosition' in self.capabilities and self.capabilities['parkingPosition'].status.value is None):
+        if (selective is None or any(x in selective for x in [Domain.ALL, Domain.PARKING])) \
+                and (not updateCapabilities or ('parkingPosition' in self.capabilities and self.capabilities['parkingPosition'].status.value is None)):
             url = 'https://mobileapi.apps.emea.vwapps.io/vehicles/' + self.vin.value + '/parkingposition'
             data = self.weConnect.fetchData(url, force, allowEmpty=True, allowHttpError=True, allowedErrors=[codes['not_found'],
                                                                                                              codes['no_content'],
@@ -385,7 +347,7 @@ class Vehicle(AddressableObject):  # pylint: disable=too-many-instance-attribute
                 if img is None or self.weConnect.maxAgePictures is None \
                         or (cacheDate is not None and cacheDate < (datetime.utcnow() - timedelta(seconds=self.weConnect.maxAgePictures))):
                     try:
-                        imageDownloadResponse = self.weConnect.session.get(imageurl, stream=True, timeout=self.weConnect.timeout)
+                        imageDownloadResponse = self.weConnect.session.get(imageurl, stream=True)
                         self.weConnect.recordElapsed(imageDownloadResponse.elapsed)
                         if imageDownloadResponse.status_code == codes['ok']:
                             img = Image.open(imageDownloadResponse.raw)
@@ -397,7 +359,7 @@ class Vehicle(AddressableObject):  # pylint: disable=too-many-instance-attribute
                         elif imageDownloadResponse.status_code == codes['unauthorized']:
                             LOG.info('Server asks for new authorization')
                             self.weConnect.login()
-                            imageDownloadResponse = self.weConnect.session.get(imageurl, stream=True, timeout=self.weConnect.timeout)
+                            imageDownloadResponse = self.weConnect.session.get(imageurl, stream=True)
                             self.weConnect.recordElapsed(imageDownloadResponse.elapsed)
                             if imageDownloadResponse.status_code == codes['ok']:
                                 img = Image.open(imageDownloadResponse.raw)
@@ -626,6 +588,8 @@ class Vehicle(AddressableObject):  # pylint: disable=too-many-instance-attribute
             for coUser in self.coUsers:
                 if coUser.enabled:
                     returnString += ''.join(['\t' + line for line in str(coUser).splitlines(True)]) + '\n'
+        if self.tags.enabled and self.tags.value:
+            returnString += 'Tags:               ' + ', '.join(self.tags.value) + '\n'
         if self.capabilities.enabled:
             returnString += f'Capabilities: {len(self.capabilities)} items\n'
             for capability in self.capabilities.values():
@@ -725,6 +689,9 @@ class Vehicle(AddressableObject):  # pylint: disable=too-many-instance-attribute
             GUEST_USER = 'GUEST_USER'
             UNKNOWN = 'UNKNOWN'
 
+            def __str__(self) -> str:
+                return self.value
+
         class EnrollmentStatus(Enum,):
             STARTED = 'STARTED'
             NOT_STARTED = 'NOT_STARTED'
