@@ -5,4 +5,5 @@ class ErrorEventType(Flag):
     HTTP = auto()
     TIMEOUT = auto()
     CONNECTION = auto()
-    ALL = HTTP | TIMEOUT | CONNECTION
+    JSON = auto()
+    ALL = HTTP | TIMEOUT | CONNECTION | JSON
